// Code generated by smithy-go-codegen DO NOT EDIT.

package apprunner

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	"github.com/aws/aws-sdk-go-v2/service/apprunner/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Returns a list of active App Runner observability configurations in your Amazon
// Web Services account. You can query the revisions for a specific configuration
// name or the revisions for all active configurations in your account. You can
// optionally query only the latest revision of each requested name. To retrieve a
// full description of a particular configuration revision, call and provide one of
// the ARNs returned by ListObservabilityConfigurations.
func (c *Client) ListObservabilityConfigurations(ctx context.Context, params *ListObservabilityConfigurationsInput, optFns ...func(*Options)) (*ListObservabilityConfigurationsOutput, error) {
	if params == nil {
		params = &ListObservabilityConfigurationsInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListObservabilityConfigurations", params, optFns, c.addOperationListObservabilityConfigurationsMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListObservabilityConfigurationsOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListObservabilityConfigurationsInput struct {

	// Set to true to list only the latest revision for each requested configuration
	// name. Set to false to list all revisions for each requested configuration name.
	// Default: true
	LatestOnly bool

	// The maximum number of results to include in each response (result page). It's
	// used for a paginated request. If you don't specify MaxResults, the request
	// retrieves all available results in a single response.
	MaxResults *int32

	// A token from a previous result page. It's used for a paginated request. The
	// request retrieves the next result page. All other parameter values must be
	// identical to the ones that are specified in the initial request. If you don't
	// specify NextToken, the request retrieves the first result page.
	NextToken *string

	// The name of the App Runner observability configuration that you want to list. If
	// specified, App Runner lists revisions that share this name. If not specified,
	// App Runner returns revisions of all active configurations.
	ObservabilityConfigurationName *string

	noSmithyDocumentSerde
}

type ListObservabilityConfigurationsOutput struct {

	// A list of summary information records for observability configurations. In a
	// paginated request, the request returns up to MaxResults records for each call.
	//
	// This member is required.
	ObservabilityConfigurationSummaryList []types.ObservabilityConfigurationSummary

	// The token that you can pass in a subsequent request to get the next result page.
	// It's returned in a paginated request.
	NextToken *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListObservabilityConfigurationsMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsAwsjson10_serializeOpListObservabilityConfigurations{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson10_deserializeOpListObservabilityConfigurations{}, middleware.After)
	if err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddComputePayloadSHA256Middleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = addHTTPSignerV4Middleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListObservabilityConfigurations(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	return nil
}

// ListObservabilityConfigurationsAPIClient is a client that implements the
// ListObservabilityConfigurations operation.
type ListObservabilityConfigurationsAPIClient interface {
	ListObservabilityConfigurations(context.Context, *ListObservabilityConfigurationsInput, ...func(*Options)) (*ListObservabilityConfigurationsOutput, error)
}

var _ ListObservabilityConfigurationsAPIClient = (*Client)(nil)

// ListObservabilityConfigurationsPaginatorOptions is the paginator options for
// ListObservabilityConfigurations
type ListObservabilityConfigurationsPaginatorOptions struct {
	// The maximum number of results to include in each response (result page). It's
	// used for a paginated request. If you don't specify MaxResults, the request
	// retrieves all available results in a single response.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListObservabilityConfigurationsPaginator is a paginator for
// ListObservabilityConfigurations
type ListObservabilityConfigurationsPaginator struct {
	options   ListObservabilityConfigurationsPaginatorOptions
	client    ListObservabilityConfigurationsAPIClient
	params    *ListObservabilityConfigurationsInput
	nextToken *string
	firstPage bool
}

// NewListObservabilityConfigurationsPaginator returns a new
// ListObservabilityConfigurationsPaginator
func NewListObservabilityConfigurationsPaginator(client ListObservabilityConfigurationsAPIClient, params *ListObservabilityConfigurationsInput, optFns ...func(*ListObservabilityConfigurationsPaginatorOptions)) *ListObservabilityConfigurationsPaginator {
	if params == nil {
		params = &ListObservabilityConfigurationsInput{}
	}

	options := ListObservabilityConfigurationsPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListObservabilityConfigurationsPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListObservabilityConfigurationsPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListObservabilityConfigurations page.
func (p *ListObservabilityConfigurationsPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListObservabilityConfigurationsOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	result, err := p.client.ListObservabilityConfigurations(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

func newServiceMetadataMiddleware_opListObservabilityConfigurations(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		SigningName:   "apprunner",
		OperationName: "ListObservabilityConfigurations",
	}
}
