// Code generated by smithy-go-codegen DO NOT EDIT.

package directoryservice

import (
	"context"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	"github.com/aws/aws-sdk-go-v2/service/directoryservice/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Retrieves information about the type of client authentication for the specified
// directory, if the type is specified. If no type is specified, information about
// all client authentication types that are supported for the specified directory
// is retrieved. Currently, only SmartCard is supported.
func (c *Client) DescribeClientAuthenticationSettings(ctx context.Context, params *DescribeClientAuthenticationSettingsInput, optFns ...func(*Options)) (*DescribeClientAuthenticationSettingsOutput, error) {
	if params == nil {
		params = &DescribeClientAuthenticationSettingsInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "DescribeClientAuthenticationSettings", params, optFns, c.addOperationDescribeClientAuthenticationSettingsMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*DescribeClientAuthenticationSettingsOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type DescribeClientAuthenticationSettingsInput struct {

	// The identifier of the directory for which to retrieve information.
	//
	// This member is required.
	DirectoryId *string

	// The maximum number of items to return. If this value is zero, the maximum number
	// of items is specified by the limitations of the operation.
	Limit *int32

	// The DescribeClientAuthenticationSettingsResult.NextToken value from a previous
	// call to DescribeClientAuthenticationSettings. Pass null if this is the first
	// call.
	NextToken *string

	// The type of client authentication for which to retrieve information. If no type
	// is specified, a list of all client authentication types that are supported for
	// the specified directory is retrieved.
	Type types.ClientAuthenticationType

	noSmithyDocumentSerde
}

type DescribeClientAuthenticationSettingsOutput struct {

	// Information about the type of client authentication for the specified directory.
	// The following information is retrieved: The date and time when the status of the
	// client authentication type was last updated, whether the client authentication
	// type is enabled or disabled, and the type of client authentication.
	ClientAuthenticationSettingsInfo []types.ClientAuthenticationSettingInfo

	// The next token used to retrieve the client authentication settings if the number
	// of setting types exceeds page limit and there is another page.
	NextToken *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationDescribeClientAuthenticationSettingsMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsAwsjson11_serializeOpDescribeClientAuthenticationSettings{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson11_deserializeOpDescribeClientAuthenticationSettings{}, middleware.After)
	if err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddComputePayloadSHA256Middleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = addHTTPSignerV4Middleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addOpDescribeClientAuthenticationSettingsValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opDescribeClientAuthenticationSettings(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opDescribeClientAuthenticationSettings(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		SigningName:   "ds",
		OperationName: "DescribeClientAuthenticationSettings",
	}
}
