// Code generated by smithy-go-codegen DO NOT EDIT.

package glacier

import (
	"context"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	glaciercust "github.com/aws/aws-sdk-go-v2/service/glacier/internal/customizations"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// This operation retrieves the following attributes from the lock-policy
// subresource set on the specified vault:
//
// * The vault lock policy set on the
// vault.
//
// * The state of the vault lock, which is either InProgess or Locked.
//
// *
// When the lock ID expires. The lock ID is used to complete the vault locking
// process.
//
// * When the vault lock was initiated and put into the InProgress
// state.
//
// A vault lock is put into the InProgress state by calling
// InitiateVaultLock. A vault lock is put into the Locked state by calling
// CompleteVaultLock. You can abort the vault locking process by calling
// AbortVaultLock. For more information about the vault locking process, Amazon
// Glacier Vault Lock
// (https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-lock.html). If there
// is no vault lock policy set on the vault, the operation returns a 404 Not found
// error. For more information about vault lock policies, Amazon Glacier Access
// Control with Vault Lock Policies
// (https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-lock-policy.html).
func (c *Client) GetVaultLock(ctx context.Context, params *GetVaultLockInput, optFns ...func(*Options)) (*GetVaultLockOutput, error) {
	if params == nil {
		params = &GetVaultLockInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "GetVaultLock", params, optFns, c.addOperationGetVaultLockMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*GetVaultLockOutput)
	out.ResultMetadata = metadata
	return out, nil
}

// The input values for GetVaultLock.
type GetVaultLockInput struct {

	// The AccountId value is the AWS account ID of the account that owns the vault.
	// You can either specify an AWS account ID or optionally a single '-' (hyphen), in
	// which case Amazon S3 Glacier uses the AWS account ID associated with the
	// credentials used to sign the request. If you use an account ID, do not include
	// any hyphens ('-') in the ID.
	//
	// This member is required.
	AccountId *string

	// The name of the vault.
	//
	// This member is required.
	VaultName *string

	noSmithyDocumentSerde
}

// Contains the Amazon S3 Glacier response to your request.
type GetVaultLockOutput struct {

	// The UTC date and time at which the vault lock was put into the InProgress state.
	CreationDate *string

	// The UTC date and time at which the lock ID expires. This value can be null if
	// the vault lock is in a Locked state.
	ExpirationDate *string

	// The vault lock policy as a JSON string, which uses "\" as an escape character.
	Policy *string

	// The state of the vault lock. InProgress or Locked.
	State *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationGetVaultLockMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsRestjson1_serializeOpGetVaultLock{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpGetVaultLock{}, middleware.After)
	if err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddComputePayloadSHA256Middleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = addHTTPSignerV4Middleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addOpGetVaultLockValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opGetVaultLock(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = glaciercust.AddTreeHashMiddleware(stack); err != nil {
		return err
	}
	if err = glaciercust.AddGlacierAPIVersionMiddleware(stack, ServiceAPIVersion); err != nil {
		return err
	}
	if err = glaciercust.AddDefaultAccountIDMiddleware(stack, setDefaultAccountID); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opGetVaultLock(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		SigningName:   "glacier",
		OperationName: "GetVaultLock",
	}
}
