// Code generated by smithy-go-codegen DO NOT EDIT.

package types

import (
	smithydocument "github.com/aws/smithy-go/document"
	"time"
)

// A signal that represents a vehicle device such as the engine, heater, and door
// locks. Data from an actuator reports the state of a certain vehicle device.
// Updating actuator data can change the state of a device. For example, you can
// turn on or off the heater by updating its actuator data.
type Actuator struct {

	// The specified data type of the actuator.
	//
	// This member is required.
	DataType NodeDataType

	// The fully qualified name of the actuator. For example, the fully qualified name
	// of an actuator might be Vehicle.Front.Left.Door.Lock.
	//
	// This member is required.
	FullyQualifiedName *string

	// A list of possible values an actuator can take.
	AllowedValues []string

	// A specified value for the actuator.
	AssignedValue *string

	// A brief description of the actuator.
	Description *string

	// The specified possible maximum value of an actuator.
	Max *float64

	// The specified possible minimum value of an actuator.
	Min *float64

	// The scientific unit for the actuator.
	Unit *string

	noSmithyDocumentSerde
}

// A signal that represents static information about the vehicle, such as engine
// type or manufacturing date.
type Attribute struct {

	// The specified data type of the attribute.
	//
	// This member is required.
	DataType NodeDataType

	// The fully qualified name of the attribute. For example, the fully qualified name
	// of an attribute might be Vehicle.Body.Engine.Type.
	//
	// This member is required.
	FullyQualifiedName *string

	// A list of possible values an attribute can be assigned.
	AllowedValues []string

	// A specified value for the attribute.
	AssignedValue *string

	// The default value of the attribute.
	DefaultValue *string

	// A brief description of the attribute.
	Description *string

	// The specified possible maximum value of the attribute.
	Max *float64

	// The specified possible minimum value of the attribute.
	Min *float64

	// The scientific unit for the attribute.
	Unit *string

	noSmithyDocumentSerde
}

// A group of signals that are defined in a hierarchical structure.
type Branch struct {

	// The fully qualified name of the branch. For example, the fully qualified name of
	// a branch might be Vehicle.Body.Engine.
	//
	// This member is required.
	FullyQualifiedName *string

	// A brief description of the branch.
	Description *string

	noSmithyDocumentSerde
}

// Information about a campaign. You can use the API operation to return this
// information about multiple created campaigns.
type CampaignSummary struct {

	// The time the campaign was created.
	//
	// This member is required.
	CreationTime *time.Time

	// The last time the campaign was modified.
	//
	// This member is required.
	LastModificationTime *time.Time

	// The Amazon Resource Name (ARN) of a campaign.
	Arn *string

	// The description of the campaign.
	Description *string

	// The name of a campaign.
	Name *string

	// The ARN of the signal catalog associated with the campaign.
	SignalCatalogArn *string

	// The state of a campaign. The status can be one of the following:
	//
	// * CREATING -
	// Amazon Web Services IoT FleetWise is processing your request to create the
	// campaign.
	//
	// * WAITING_FOR_APPROVAL - After a campaign is created, it enters the
	// WAITING_FOR_APPROVAL state. To allow Amazon Web Services IoT FleetWise to deploy
	// the campaign to the target vehicle or fleet, use the API operation to approve
	// the campaign.
	//
	// * RUNNING - The campaign is active.
	//
	// * SUSPENDED - The campaign
	// is suspended. To resume the campaign, use the API operation.
	Status CampaignStatus

	// The ARN of a vehicle or fleet to which the campaign is deployed.
	TargetArn *string

	noSmithyDocumentSerde
}

// Configurations used to create a decoder manifest.
type CanDbcDefinition struct {

	// A list of DBC files. You can upload only one DBC file for each network interface
	// and specify up to five (inclusive) files in the list.
	//
	// This member is required.
	CanDbcFiles [][]byte

	// Contains information about a network interface.
	//
	// This member is required.
	NetworkInterface *string

	// Pairs every signal specified in your vehicle model with a signal decoder.
	SignalsMap map[string]string

	noSmithyDocumentSerde
}

// A single controller area network (CAN) device interface.
type CanInterface struct {

	// The unique name of the interface.
	//
	// This member is required.
	Name *string

	// The name of the communication protocol for the interface.
	ProtocolName *string

	// The version of the communication protocol for the interface.
	ProtocolVersion *string

	noSmithyDocumentSerde
}

// Information about a single controller area network (CAN) signal and the messages
// it receives and transmits.
type CanSignal struct {

	// A multiplier used to decode the CAN message.
	//
	// This member is required.
	Factor *float64

	// Whether the byte ordering of a CAN message is big-endian.
	//
	// This member is required.
	IsBigEndian bool

	// Whether the message data is specified as a signed value.
	//
	// This member is required.
	IsSigned bool

	// How many bytes of data are in the message.
	//
	// This member is required.
	Length int32

	// The ID of the message.
	//
	// This member is required.
	MessageId int32

	// Indicates where data appears in the CAN message.
	//
	// This member is required.
	Offset *float64

	// Indicates the beginning of the CAN message.
	//
	// This member is required.
	StartBit int32

	// The name of the signal.
	Name *string

	noSmithyDocumentSerde
}

// The log delivery option to send data to Amazon CloudWatch Logs.
type CloudWatchLogDeliveryOptions struct {

	// The type of log to send data to Amazon CloudWatch Logs.
	//
	// This member is required.
	LogType LogType

	// The Amazon CloudWatch Logs group the operation sends data to.
	LogGroupName *string

	noSmithyDocumentSerde
}

// Specifies what data to collect and how often or when to collect it.
//
// The following types satisfy this interface:
//
//	CollectionSchemeMemberConditionBasedCollectionScheme
//	CollectionSchemeMemberTimeBasedCollectionScheme
type CollectionScheme interface {
	isCollectionScheme()
}

// Information about a collection scheme that uses a simple logical expression to
// recognize what data to collect.
type CollectionSchemeMemberConditionBasedCollectionScheme struct {
	Value ConditionBasedCollectionScheme

	noSmithyDocumentSerde
}

func (*CollectionSchemeMemberConditionBasedCollectionScheme) isCollectionScheme() {}

// Information about a collection scheme that uses a time period to decide how
// often to collect data.
type CollectionSchemeMemberTimeBasedCollectionScheme struct {
	Value TimeBasedCollectionScheme

	noSmithyDocumentSerde
}

func (*CollectionSchemeMemberTimeBasedCollectionScheme) isCollectionScheme() {}

// Information about a collection scheme that uses a simple logical expression to
// recognize what data to collect.
type ConditionBasedCollectionScheme struct {

	// The logical expression used to recognize what data to collect. For example,
	// $variable.Vehicle.OutsideAirTemperature >= 105.0.
	//
	// This member is required.
	Expression *string

	// Specifies the version of the conditional expression language.
	ConditionLanguageVersion *int32

	// The minimum duration of time between two triggering events to collect data, in
	// milliseconds. If a signal changes often, you might want to collect data at a
	// slower rate.
	MinimumTriggerIntervalMs *int64

	// Whether to collect data for all triggering events (ALWAYS). Specify
	// (RISING_EDGE), or specify only when the condition first evaluates to false. For
	// example, triggering on "AirbagDeployed"; Users aren't interested on triggering
	// when the airbag is already exploded; they only care about the change from not
	// deployed => deployed.
	TriggerMode TriggerMode

	noSmithyDocumentSerde
}

// An HTTP error resulting from creating a vehicle.
type CreateVehicleError struct {

	// An HTTP error code.
	Code *string

	// A description of the HTTP error.
	Message *string

	// The ID of the vehicle with the error.
	VehicleName *string

	noSmithyDocumentSerde
}

// Information about the vehicle to create.
type CreateVehicleRequestItem struct {

	// The Amazon Resource Name (ARN) of a decoder manifest associated with the vehicle
	// to create.
	//
	// This member is required.
	DecoderManifestArn *string

	// The ARN of the vehicle model (model manifest) to create the vehicle from.
	//
	// This member is required.
	ModelManifestArn *string

	// The unique ID of the vehicle to create.
	//
	// This member is required.
	VehicleName *string

	// An option to create a new Amazon Web Services IoT thing when creating a vehicle,
	// or to validate an existing thing as a vehicle.
	AssociationBehavior VehicleAssociationBehavior

	// Static information about a vehicle in a key-value pair. For example: "engine
	// Type" : "v6"
	Attributes map[string]string

	// Metadata which can be used to manage the vehicle.
	Tags []Tag

	noSmithyDocumentSerde
}

// Information about a created vehicle.
type CreateVehicleResponseItem struct {

	// The ARN of the created vehicle.
	Arn *string

	// The ARN of a created or validated Amazon Web Services IoT thing.
	ThingArn *string

	// The unique ID of the vehicle to create.
	VehicleName *string

	noSmithyDocumentSerde
}

// Information about a created decoder manifest. You can use the API operation to
// return this information about multiple decoder manifests.
type DecoderManifestSummary struct {

	// The time the decoder manifest was created in seconds since epoch (January 1,
	// 1970 at midnight UTC time).
	//
	// This member is required.
	CreationTime *time.Time

	// The time the decoder manifest was last updated in seconds since epoch (January
	// 1, 1970 at midnight UTC time).
	//
	// This member is required.
	LastModificationTime *time.Time

	// The ARN of a vehicle model (model manifest) associated with the decoder
	// manifest.
	Arn *string

	// A brief description of the decoder manifest.
	Description *string

	// The ARN of a vehicle model (model manifest) associated with the decoder
	// manifest.
	ModelManifestArn *string

	// The name of the decoder manifest.
	Name *string

	// The state of the decoder manifest. If the status is ACTIVE, the decoder manifest
	// can't be edited. If the status is marked DRAFT, you can edit the decoder
	// manifest.
	Status ManifestStatus

	noSmithyDocumentSerde
}

// Information about a fleet. You can use the API operation to return this
// information about multiple fleets.
type FleetSummary struct {

	// The Amazon Resource Name (ARN) of the fleet.
	//
	// This member is required.
	Arn *string

	// The time the fleet was created, in seconds since epoch (January 1, 1970 at
	// midnight UTC time).
	//
	// This member is required.
	CreationTime *time.Time

	// The unique ID of the fleet.
	//
	// This member is required.
	Id *string

	// The ARN of the signal catalog associated with the fleet.
	//
	// This member is required.
	SignalCatalogArn *string

	// A brief description of the fleet.
	Description *string

	// The time the fleet was last updated in seconds since epoch (January 1, 1970 at
	// midnight UTC time).
	LastModificationTime *time.Time

	noSmithyDocumentSerde
}

// Vehicle Signal Specification (VSS) is a precise language used to describe and
// model signals in vehicle networks. The JSON file collects signal specificiations
// in a VSS format.
//
// The following types satisfy this interface:
//
//	FormattedVssMemberVssJson
type FormattedVss interface {
	isFormattedVss()
}

// Provides the VSS in JSON format.
type FormattedVssMemberVssJson struct {
	Value string

	noSmithyDocumentSerde
}

func (*FormattedVssMemberVssJson) isFormattedVss() {}

// Information about registering an Identity and Access Management (IAM) resource
// so Amazon Web Services IoT FleetWise edge agent software can transfer your
// vehicle data to Amazon Timestream.
type IamRegistrationResponse struct {

	// The status of registering your IAM resource. The status can be one of
	// REGISTRATION_SUCCESS, REGISTRATION_PENDING, REGISTRATION_FAILURE.
	//
	// This member is required.
	RegistrationStatus RegistrationStatus

	// The Amazon Resource Name (ARN) of the IAM role to register.
	//
	// This member is required.
	RoleArn *string

	// A message associated with a registration error.
	ErrorMessage *string

	noSmithyDocumentSerde
}

// The IAM resource that enables Amazon Web Services IoT FleetWise edge agent
// software to send data to Amazon Timestream. For more information, see IAM roles
// (https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles.html) in the Identity
// and Access Management User Guide.
type IamResources struct {

	// The Amazon Resource Name (ARN) of the IAM resource that allows Amazon Web
	// Services IoT FleetWise to send data to Amazon Timestream. For example,
	// arn:aws:iam::123456789012:role/SERVICE-ROLE-ARN.
	//
	// This member is required.
	RoleArn *string

	noSmithyDocumentSerde
}

// A reason a vehicle network interface isn't valid.
type InvalidNetworkInterface struct {

	// The ID of the interface that isn't valid.
	InterfaceId *string

	// A message about why the interface isn't valid.
	Reason NetworkInterfaceFailureReason

	noSmithyDocumentSerde
}

// A reason that a signal isn't valid.
type InvalidSignal struct {

	// The name of the signal that isn't valid.
	Name *string

	// A message about why the signal isn't valid.
	Reason *string

	noSmithyDocumentSerde
}

// A reason that a signal decoder isn't valid.
type InvalidSignalDecoder struct {

	// The name of a signal decoder that isn't valid.
	Name *string

	// A message about why the signal decoder isn't valid.
	Reason SignalDecoderFailureReason

	noSmithyDocumentSerde
}

// Information about a vehicle model (model manifest). You can use the API
// operation to return this information about multiple vehicle models.
type ModelManifestSummary struct {

	// The time the vehicle model was created, in seconds since epoch (January 1, 1970
	// at midnight UTC time).
	//
	// This member is required.
	CreationTime *time.Time

	// The time the vehicle model was last updated, in seconds since epoch (January 1,
	// 1970 at midnight UTC time).
	//
	// This member is required.
	LastModificationTime *time.Time

	// The Amazon Resource Name (ARN) of the vehicle model.
	Arn *string

	// A brief description of the vehicle model.
	Description *string

	// The name of the vehicle model.
	Name *string

	// The ARN of the signal catalog associated with the vehicle model.
	SignalCatalogArn *string

	// The state of the vehicle model. If the status is ACTIVE, the vehicle model can't
	// be edited. If the status is DRAFT, you can edit the vehicle model.
	Status ManifestStatus

	noSmithyDocumentSerde
}

// Specifications for defining a vehicle network.
//
// The following types satisfy this interface:
//
//	NetworkFileDefinitionMemberCanDbc
type NetworkFileDefinition interface {
	isNetworkFileDefinition()
}

// Information, including CAN DBC files, about the configurations used to create a
// decoder manifest.
type NetworkFileDefinitionMemberCanDbc struct {
	Value CanDbcDefinition

	noSmithyDocumentSerde
}

func (*NetworkFileDefinitionMemberCanDbc) isNetworkFileDefinition() {}

// Represents a node and its specifications in an in-vehicle communication network.
// All signal decoders must be associated with a network node. To return this
// information about all the network interfaces specified in a decoder manifest,
// use the API operation.
type NetworkInterface struct {

	// The ID of the network interface.
	//
	// This member is required.
	InterfaceId *string

	// The network protocol for the vehicle. For example, CAN_SIGNAL specifies a
	// protocol that defines how data is communicated between electronic control units
	// (ECUs). OBD_SIGNAL specifies a protocol that defines how self-diagnostic data is
	// communicated between ECUs.
	//
	// This member is required.
	Type NetworkInterfaceType

	// Information about a network interface specified by the Controller Area Network
	// (CAN) protocol.
	CanInterface *CanInterface

	// Information about a network interface specified by the On-board diagnostic (OBD)
	// II protocol.
	ObdInterface *ObdInterface

	noSmithyDocumentSerde
}

// A general abstraction of a signal. A node can be specified as an actuator,
// attribute, branch, or sensor.
//
// The following types satisfy this interface:
//
//	NodeMemberActuator
//	NodeMemberAttribute
//	NodeMemberBranch
//	NodeMemberSensor
type Node interface {
	isNode()
}

// Information about a node specified as an actuator. An actuator is a digital
// representation of a vehicle device.
type NodeMemberActuator struct {
	Value Actuator

	noSmithyDocumentSerde
}

func (*NodeMemberActuator) isNode() {}

// Information about a node specified as an attribute. An attribute represents
// static information about a vehicle.
type NodeMemberAttribute struct {
	Value Attribute

	noSmithyDocumentSerde
}

func (*NodeMemberAttribute) isNode() {}

// Information about a node specified as a branch. A group of signals that are
// defined in a hierarchical structure.
type NodeMemberBranch struct {
	Value Branch

	noSmithyDocumentSerde
}

func (*NodeMemberBranch) isNode() {}

// An input component that reports the environmental condition of a vehicle. You
// can collect data about fluid levels, temperatures, vibrations, or battery
// voltage from sensors.
type NodeMemberSensor struct {
	Value Sensor

	noSmithyDocumentSerde
}

func (*NodeMemberSensor) isNode() {}

// Information about the number of nodes and node types in a vehicle network.
type NodeCounts struct {

	// The total number of nodes in a vehicle network that represent actuators.
	TotalActuators int32

	// The total number of nodes in a vehicle network that represent attributes.
	TotalAttributes int32

	// The total number of nodes in a vehicle network that represent branches.
	TotalBranches int32

	// The total number of nodes in a vehicle network.
	TotalNodes int32

	// The total number of nodes in a vehicle network that represent sensors.
	TotalSensors int32

	noSmithyDocumentSerde
}

// A network interface that specifies the On-board diagnostic (OBD) II network
// protocol.
type ObdInterface struct {

	// The name of the interface.
	//
	// This member is required.
	Name *string

	// The ID of the message requesting vehicle data.
	//
	// This member is required.
	RequestMessageId int32

	// The maximum number message requests per diagnostic trouble code per second.
	DtcRequestIntervalSeconds int32

	// Whether the vehicle has a transmission control module (TCM).
	HasTransmissionEcu bool

	// The standard OBD II PID.
	ObdStandard *string

	// The maximum number message requests per second.
	PidRequestIntervalSeconds int32

	// Whether to use extended IDs in the message.
	UseExtendedIds bool

	noSmithyDocumentSerde
}

// Information about signal messages using the on-board diagnostics (OBD) II
// protocol in a vehicle.
type ObdSignal struct {

	// The length of a message.
	//
	// This member is required.
	ByteLength *int32

	// Indicates where data appears in the message.
	//
	// This member is required.
	Offset *float64

	// The diagnostic code used to request data from a vehicle for this signal.
	//
	// This member is required.
	Pid int32

	// The length of the requested data.
	//
	// This member is required.
	PidResponseLength int32

	// A multiplier used to decode the message.
	//
	// This member is required.
	Scaling *float64

	// The mode of operation (diagnostic service) in a message.
	//
	// This member is required.
	ServiceMode int32

	// Indicates the beginning of the message.
	//
	// This member is required.
	StartByte int32

	// The number of bits to mask in a message.
	BitMaskLength *int32

	// The number of positions to shift bits in the message.
	BitRightShift int32

	noSmithyDocumentSerde
}

// An input component that reports the environmental condition of a vehicle. You
// can collect data about fluid levels, temperatures, vibrations, or battery
// voltage from sensors.
type Sensor struct {

	// The specified data type of the sensor.
	//
	// This member is required.
	DataType NodeDataType

	// The fully qualified name of the sensor. For example, the fully qualified name of
	// a sensor might be Vehicle.Body.Engine.Battery.
	//
	// This member is required.
	FullyQualifiedName *string

	// A list of possible values a sensor can take.
	AllowedValues []string

	// A brief description of a sensor.
	Description *string

	// The specified possible maximum value of the sensor.
	Max *float64

	// The specified possible minimum value of the sensor.
	Min *float64

	// The scientific unit of measurement for data collected by the sensor.
	Unit *string

	noSmithyDocumentSerde
}

// Information about a collection of standardized signals, which can be attributes,
// branches, sensors, or actuators.
type SignalCatalogSummary struct {

	// The Amazon Resource Name (ARN) of the signal catalog.
	Arn *string

	// The time the signal catalog was created in seconds since epoch (January 1, 1970
	// at midnight UTC time).
	CreationTime *time.Time

	// The time the signal catalog was last updated in seconds since epoch (January 1,
	// 1970 at midnight UTC time).
	LastModificationTime *time.Time

	// The name of the signal catalog.
	Name *string

	noSmithyDocumentSerde
}

// Information about a signal decoder.
type SignalDecoder struct {

	// The fully qualified name of a signal decoder as defined in a vehicle model.
	//
	// This member is required.
	FullyQualifiedName *string

	// The ID of a network interface that specifies what network protocol a vehicle
	// follows.
	//
	// This member is required.
	InterfaceId *string

	// The network protocol for the vehicle. For example, CAN_SIGNAL specifies a
	// protocol that defines how data is communicated between electronic control units
	// (ECUs). OBD_SIGNAL specifies a protocol that defines how self-diagnostic data is
	// communicated between ECUs.
	//
	// This member is required.
	Type SignalDecoderType

	// Information about signal decoder using the Controller Area Network (CAN)
	// protocol.
	CanSignal *CanSignal

	// Information about signal decoder using the On-board diagnostic (OBD) II
	// protocol.
	ObdSignal *ObdSignal

	noSmithyDocumentSerde
}

// Information about a signal.
type SignalInformation struct {

	// The name of the signal.
	//
	// This member is required.
	Name *string

	// The maximum number of samples to collect.
	MaxSampleCount *int64

	// The minimum duration of time (in milliseconds) between two triggering events to
	// collect data. If a signal changes often, you might want to collect data at a
	// slower rate.
	MinimumSamplingIntervalMs *int64

	noSmithyDocumentSerde
}

// A set of key/value pairs that are used to manage the resource.
type Tag struct {

	// The tag's key.
	//
	// This member is required.
	Key *string

	// The tag's value.
	//
	// This member is required.
	Value *string

	noSmithyDocumentSerde
}

// Information about a collection scheme that uses a time period to decide how
// often to collect data.
type TimeBasedCollectionScheme struct {

	// The time period (in milliseconds) to decide how often to collect data. For
	// example, if the time period is 60000, the Edge Agent software collects data once
	// every minute.
	//
	// This member is required.
	PeriodMs *int64

	noSmithyDocumentSerde
}

// Information about the registered Amazon Timestream resources or errors, if any.
type TimestreamRegistrationResponse struct {

	// The status of registering your Amazon Timestream resources. The status can be
	// one of REGISTRATION_SUCCESS, REGISTRATION_PENDING, REGISTRATION_FAILURE.
	//
	// This member is required.
	RegistrationStatus RegistrationStatus

	// The name of the Timestream database.
	//
	// This member is required.
	TimestreamDatabaseName *string

	// The name of the Timestream database table.
	//
	// This member is required.
	TimestreamTableName *string

	// A message associated with a registration error.
	ErrorMessage *string

	// The Amazon Resource Name (ARN) of the Timestream database.
	TimestreamDatabaseArn *string

	// The ARN of the Timestream database table.
	TimestreamTableArn *string

	noSmithyDocumentSerde
}

// The registered Amazon Timestream resources that Amazon Web Services IoT
// FleetWise edge agent software can transfer your vehicle data to.
type TimestreamResources struct {

	// The name of the registered Amazon Timestream database.
	//
	// This member is required.
	TimestreamDatabaseName *string

	// The name of the registered Amazon Timestream database table.
	//
	// This member is required.
	TimestreamTableName *string

	noSmithyDocumentSerde
}

// An HTTP error resulting from updating the description for a vehicle.
type UpdateVehicleError struct {

	// The relevant HTTP error code (400+).
	Code int32

	// A message associated with the error.
	Message *string

	// The ID of the vehicle with the error.
	VehicleName *string

	noSmithyDocumentSerde
}

// Information about the vehicle to update.
type UpdateVehicleRequestItem struct {

	// The unique ID of the vehicle to update.
	//
	// This member is required.
	VehicleName *string

	// The method the specified attributes will update the existing attributes on the
	// vehicle. UseOverwite to replace the vehicle attributes with the specified
	// attributes. Or use Merge to combine all attributes. This is required if
	// attributes are present in the input.
	AttributeUpdateMode UpdateMode

	// Static information about a vehicle in a key-value pair. For example:
	// "engineType" : "1.3 L R2"
	Attributes map[string]string

	// The ARN of the signal decoder manifest associated with the vehicle to update.
	DecoderManifestArn *string

	// The ARN of the vehicle model (model manifest) associated with the vehicle to
	// update.
	ModelManifestArn *string

	noSmithyDocumentSerde
}

// Information about the updated vehicle.
type UpdateVehicleResponseItem struct {

	// The Amazon Resource Name (ARN) of the updated vehicle.
	Arn *string

	// The unique ID of the updated vehicle.
	VehicleName *string

	noSmithyDocumentSerde
}

// A validation error due to mismatch between the expected data type, length, or
// pattern of the parameter and the input.
type ValidationExceptionField struct {

	// A message about the validation error.
	//
	// This member is required.
	Message *string

	// The name of the parameter field with the validation error.
	//
	// This member is required.
	Name *string

	noSmithyDocumentSerde
}

// Information about the state of a vehicle and how it relates to the status of a
// campaign.
type VehicleStatus struct {

	// The name of a campaign.
	CampaignName *string

	// The state of a vehicle, which can be one of the following:
	//
	// * CREATED - Amazon
	// Web Services IoT FleetWise sucessfully created the vehicle.
	//
	// * READY - The
	// vehicle is ready to receive a campaign deployment.
	//
	// * HEALTHY - A campaign
	// deployment was delivered to the vehicle.
	//
	// * SUSPENDED - A campaign associated
	// with the vehicle was suspended and data collection was paused.
	//
	// * DELETING -
	// Amazon Web Services IoT FleetWise is removing a campaign from the vehicle.
	Status VehicleState

	// The unique ID of the vehicle.
	VehicleName *string

	noSmithyDocumentSerde
}

// Information about a vehicle. To return this information about vehicles in your
// account, you can use the API operation.
type VehicleSummary struct {

	// The Amazon Resource Name (ARN) of the vehicle.
	//
	// This member is required.
	Arn *string

	// The time the vehicle was created in seconds since epoch (January 1, 1970 at
	// midnight UTC time).
	//
	// This member is required.
	CreationTime *time.Time

	// The ARN of a decoder manifest associated with the vehicle.
	//
	// This member is required.
	DecoderManifestArn *string

	// The time the vehicle was last updated in seconds since epoch (January 1, 1970 at
	// midnight UTC time).
	//
	// This member is required.
	LastModificationTime *time.Time

	// The ARN of a vehicle model (model manifest) associated with the vehicle.
	//
	// This member is required.
	ModelManifestArn *string

	// The unique ID of the vehicle.
	//
	// This member is required.
	VehicleName *string

	noSmithyDocumentSerde
}

type noSmithyDocumentSerde = smithydocument.NoSerde

// UnknownUnionMember is returned when a union member is returned over the wire,
// but has an unknown tag.
type UnknownUnionMember struct {
	Tag   string
	Value []byte

	noSmithyDocumentSerde
}

func (*UnknownUnionMember) isCollectionScheme()      {}
func (*UnknownUnionMember) isFormattedVss()          {}
func (*UnknownUnionMember) isNetworkFileDefinition() {}
func (*UnknownUnionMember) isNode()                  {}
